/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-11-28 16:17:55.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.print.common.utils;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

/**
 * @author: fengdesheng
 * @since: 2018/11/28
 */
public class ElectronUtils {
    private static final Logger log = LoggerFactory.getLogger(ElectronUtils.class);
    private static final String WIN_PREFIX = "cmd /c ";

    /**
     * 将URL响应内容转为PDF文件
     */
    public static boolean toPdf(String url, String path) {
        if (StringUtils.isBlank(url) || StringUtils.isBlank(path)) {
            return false;
        }

        String cmd = String.format("electron-pdf %s %s -p A3 -l true -e view-ready", url, path);
        if (isWindows()) {
            cmd = WIN_PREFIX + cmd;
        }

        return executeCommand(cmd);
    }

    /**
     * 执行命令
     */
    private static boolean executeCommand(String cmd) {
        Process process;

        try {
            process = Runtime.getRuntime().exec(cmd);
        } catch (IOException e) {
            log.error(e.getMessage());
            return false;
        }

        String charsetName = "UTF-8";
        if (isWindows()) {
            charsetName = "GBK";
        }

        try (BufferedReader out = new BufferedReader(new InputStreamReader(process.getInputStream(), charsetName));
             BufferedReader err = new BufferedReader(new InputStreamReader(process.getErrorStream(), charsetName));) {

            String outStr;
            while ((outStr = out.readLine()) != null) {
                log.debug(outStr);
            }

            String errStr;
            while ((errStr = err.readLine()) != null) {
                log.debug("err:" + errStr);
            }

            int exitCode = process.waitFor();
            if (exitCode == 0) {
                log.debug("Execute Command Success.");
                return true;
            }

            log.warn("Execute Command Error. exitCode:" + exitCode);
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        } catch (InterruptedException e) {
            log.error(e.getMessage(), e);
        } finally {
            if (process != null) {
                process.destroy();
            }
        }
        return false;
    }

    /**
     * 判断是否为Windows系统
     */
    private static boolean isWindows() {
        String os = System.getProperty("os.name").toLowerCase();
        if (os.contains("windows") || os.startsWith("win")) {
            return true;
        }
        return false;
    }

}